@@ -9,21 +9,21 @@
 // or else upgrading old project file will not work
 // reliably.
 // TODO QGIS 4: remove in favor of _QGIS_VERSION
-#define VERSION "3.10.2-A Coruña"
-#define _QGIS_VERSION "3.10.2-A Coruña"
+#define VERSION "3.12.3-București"
+#define _QGIS_VERSION "3.12.3-București"
 
 //used in vim src/core/qgis.cpp
 //The way below should work but it resolves to a number like 0110 which the compiler treats as octal I think
 //because debuggin it out shows the decimal number 72 which results in incorrect version status.
 //As a short term fix I (Tim) am defining the version in top level cmake. It would be good to
 //reinstate this more generic approach below at some point though
-//#define VERSION_INT 3102
+//#define VERSION_INT 3123
 // TODO QGIS 4: Remove in favor of _QGIS_VERSION_INT
-#define VERSION_INT 31002
-#define _QGIS_VERSION_INT 31002
-#define ABISYM(x) x ## 31002
+#define VERSION_INT 31203
+#define _QGIS_VERSION_INT 31203
+#define ABISYM(x) x ## 31203
 //used in main.cpp and anywhere else where the release name is needed
-#define RELEASE_NAME "A Coruña"
+#define RELEASE_NAME "București"
 
 #define QGIS_PLUGIN_SUBDIR "lib/qgis/plugins"
 #define QGIS_DATA_SUBDIR "share/qgis"
@@ -31,7 +31,7 @@
 #define QGIS_LIB_SUBDIR "lib"
 #define QGIS_QML_SUBDIR "qml"
 #define CMAKE_INSTALL_PREFIX "/usr"
-#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.10.2"
+#define CMAKE_SOURCE_DIR "/build/qgis/src/qgis-3.12.3"
 
 #define QGIS_SERVER_MODULE_SUBDIR "lib/qgis/server"
 
