@@ -87,6 +87,29 @@ class CORE_EXPORT QgsRasterRenderer : public QgsRasterInterface
      */
     double opacity() const { return mOpacity; }
 
+    /**
+     * Returns the color to use for shading nodata pixels.
+     *
+     * If the returned value is an invalid color then the default transparent rendering of
+     * nodata values will be used.
+     *
+     * \see renderColorForNodataPixel()
+     * \see setNodataColor()
+     * \since QGIS 3.12
+     */
+    QColor nodataColor() const { return mNodataColor; }
+
+    /**
+     * Sets the \a color to use for shading nodata pixels.
+     *
+     * If \a color is an invalid color then the default transparent rendering of
+     * nodata values will be used.
+     *
+     * \see nodataColor()
+     * \since QGIS 3.12
+     */
+    void setNodataColor( const QColor &color ) { mNodataColor = color; }
+
     void setRasterTransparency( QgsRasterTransparency *t SIP_TRANSFER );
     const QgsRasterTransparency *rasterTransparency() const { return mRasterTransparency; }
 
@@ -150,7 +173,20 @@ class CORE_EXPORT QgsRasterRenderer : public QgsRasterInterface
     //! Origin of min/max values
     QgsRasterMinMaxOrigin mMinMaxOrigin;
 
+    /**
+     * Returns the color for the renderer to use to represent nodata pixels.
+     *
+     * Subclasses should use this rather then nodataColor() to determine the color to use for nodata pixels
+     * during an actual rendering operation.
+     *
+     * \since QGIS 3.10
+     */
+    QRgb renderColorForNodataPixel() const;
+
   private:
+
+    QColor mNodataColor;
+
 #ifdef SIP_RUN
     QgsRasterRenderer( const QgsRasterRenderer & );
     const QgsRasterRenderer &operator=( const QgsRasterRenderer & );
