@@ -21,10 +21,6 @@ __author__ = 'Piotr Pociask'
 __date__ = 'March 2013'
 __copyright__ = '(C) 2013, Piotr Pociask'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = 'f6ddc62fdb8ae589d3827e84d0c25bf669931040'
-
 from qgis.PyQt.QtCore import QCoreApplication
 from qgis.core import (QgsFields,
                        QgsFeature,
@@ -76,7 +72,7 @@ class Polygonize(QgisAlgorithm):
         if source is None:
             raise QgsProcessingException(self.invalidSourceError(parameters, self.INPUT))
 
-        if self.parameterAsBool(parameters, self.KEEP_FIELDS, context):
+        if self.parameterAsBoolean(parameters, self.KEEP_FIELDS, context):
             fields = source.fields()
         else:
             fields = QgsFields()
