@@ -21,10 +21,6 @@ __author__ = 'Victor Olaya'
 __date__ = 'November 2016'
 __copyright__ = '(C) 2016, Victor Olaya'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '0c5774c0685d4b6f3e92a086234455708d1cd427'
-
 import os
 import math
 from processing.algs.qgis.QgisAlgorithm import QgisAlgorithm
@@ -127,7 +123,7 @@ class RasterCalculator(QgisAlgorithm):
                 bbox = transform.transformBoundingBox(bbox)
 
         if bbox.isNull() and layers:
-            bbox = QgsProcessingUtils.combineLayerExtents(layers, crs)
+            bbox = QgsProcessingUtils.combineLayerExtents(layers, crs, context)
 
         cellsize = self.parameterAsDouble(parameters, self.CELLSIZE, context)
         if cellsize == 0 and not layers:
@@ -184,11 +180,14 @@ class RasterCalculator(QgisAlgorithm):
                                    crs,
                                    width,
                                    height,
-                                   entries)
+                                   entries,
+                                   context.transformContext())
 
         res = calc.processCalculation(feedback)
         if res == QgsRasterCalculator.ParserError:
             raise QgsProcessingException(self.tr("Error parsing formula"))
+        elif res == QgsRasterCalculator.CalculationError:
+            raise QgsProcessingException(self.tr("An error occurred while performing the calculation"))
 
         return {self.OUTPUT: output}
 
