@@ -21,10 +21,6 @@ __author__ = 'Alexander Bruy'
 __date__ = 'October 2016'
 __copyright__ = '(C) 2016, Alexander Bruy'
 
-# This will get replaced with a git SHA1 when you do a git archive
-
-__revision__ = '93e761dd2396af21db9bb93e26d7cba680c906ca'
-
 import os
 
 from qgis.PyQt.QtGui import QIcon
@@ -145,19 +141,23 @@ class TinInterpolation(QgisAlgorithm):
         layerData = []
         layers = []
         crs = QgsCoordinateReferenceSystem()
-        for row in interpolationData.split('::|::'):
+        for i, row in enumerate(interpolationData.split('::|::')):
             v = row.split('::~::')
             data = QgsInterpolator.LayerData()
 
             # need to keep a reference until interpolation is complete
             layer = QgsProcessingUtils.variantToSource(v[0], context)
             data.source = layer
+            data.transformContext = context.transformContext()
             layers.append(layer)
             if not crs.isValid():
                 crs = layer.sourceCrs()
 
             data.valueSource = int(v[1])
             data.interpolationAttribute = int(v[2])
+            if data.valueSource == QgsInterpolator.ValueAttribute and data.interpolationAttribute == -1:
+                raise QgsProcessingException(self.tr('Layer {} is set to use a value attribute, but no attribute was set'.format(i + 1)))
+
             if v[3] == '0':
                 data.sourceType = QgsInterpolator.SourcePoints
             elif v[3] == '1':
